@@ -2,7 +2,7 @@
 * Ipq Berkeley db Daemon  BAN - ibd-ban
 * written by ale in milano on 13 sep 2008
 
-Copyright (C) 2008-2011 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -20,7 +20,6 @@ You should have received a copy of the GNU General Public License
 along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 */
-#define _GNU_SOURCE 1
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,8 +29,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <signal.h>
 #include <syslog.h>
 
-#define __USE_BSD 1
-// for u_int in db.h
 #include <sys/types.h>
 
 // Berkeley DB v4.4
@@ -59,6 +56,8 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 static app_private ap;
 static char const err_prefix[] = "ibd-ban";
+#include "logline_written_block.h"
+
 int caught_signal;
 
 #if !defined NDEBUG
@@ -118,7 +117,7 @@ static int setsigs(void)
 	return rtc;
 }
 
-static int initial_count = 5;
+static int initial_count = IPQBDB_INITIAL_COUNT;
 static double initial_decay = IPQBDB_INITIAL_DECAY;
 static char *db_block_name = IPQBDB_DATABASE_NAME;
 static char *db_white_name = IPQBDB_WHITE_DATABASE_NAME;
@@ -278,6 +277,10 @@ static void tab_printtime(char const *hdr, time_t ref, time_t now)
 static void display_written_block(uint32_t ip_addr,
 	ip_data_t *ip_data, ip_data_t *old_data, DB *db_d)
 {
+	// if syslog_opt, log a line anyway (unlikely given -v).
+	if (ap.mode)
+		logline_written_block(ip_addr, ip_data, old_data, db_d);
+
 	struct in_addr in;
 	char descr[19 /* IPQBDB_DESCR_MAX_LENGTH */];
 	memcpy(&in, &ip_addr, sizeof in);
@@ -347,8 +350,9 @@ write_block_display(DB *db, uint32_t ip_addr, double decay, int reason_id, DB *d
 	if (force_decay_opt) force |= write_block_force_decay;
 	if (force_reason_opt) force |= write_block_force_reason;
 
-	if (write_block(db, ip_addr, decay, reason_id, &probability,
-		force, verbose? display_written_block: NULL, db_d))
+	if (write_block(db, ip_addr, decay, reason_id, &probability, force,
+		verbose? display_written_block:
+		ap.mode? logline_written_block: NULL, db_d))
 			return -1;
 
 	return probability >= RAND_MAX;
@@ -391,7 +395,8 @@ int main(int argc, char const *argv[])
 	{
 		if (version_opt)
 		{
-			fprintf(stdout, "%s: version " PACKAGE_VERSION "\n", err_prefix);
+			fprintf(stdout, "%s: version " PACKAGE_VERSION "\n"
+			DB_VERSION_STRING "\n", err_prefix);
 			errs = 2;
 		}
 
