@@ -2,7 +2,7 @@
 * ipq berkeley db daemon emitting verdicts - ibd-judge
 * written by ale in milano on 10 sep 2008
 
-Copyright (C) 2008-2011 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -33,8 +33,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <ctype.h>
 #include <limits.h>
 
-#define __USE_BSD 1
-// for u_int in db.h
 #include <sys/types.h>
 
 // misc. ip stuff
@@ -145,7 +143,7 @@ typedef struct queue_descr
 	uint32_t deadbeef;
 
 	// values set by parse_queue_descr
-	uint32_t mark_value;
+	uint32_t mark_value;  // host order
 	unsigned int queue_maxlen;
 	uint16_t queue;
 	unsigned char dest;
@@ -159,6 +157,7 @@ typedef struct pkt_data
 	time_t now;
 	uint32_t ipaddr;
 	uint32_t id;
+	uint32_t old_mark;
 } pkt_data;
 
 // seems a callback may be called multiple times for a given packet...(?)
@@ -199,7 +198,7 @@ static int parse_queue_descr(char const *descr, queue_descr* out)
 				l = strtoul(descr, &t, 0);
 				if (l <= UINT32_MAX)
 				{
-					q.mark_value = htonl((uint32_t)l);
+					q.mark_value = (uint32_t)l; // was htonl in 0.4
 					q.mark = 1;
 				}
 				else
@@ -239,7 +238,7 @@ packet_cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
 {
 	(void)qh; (void)nfmsg;
 	queue_descr *qu = (queue_descr*)data;
-	ipaddr_cnt *cnt;
+	ipaddr_cnt *cnt /* compiler happy */ = NULL;
 	
 	if (qu == NULL || qu->deadbeef != 0xdeadbeef ||
 		(cnt = qu->cnt) == NULL ||
@@ -250,7 +249,7 @@ packet_cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
 		return -1;
 	}
 
-	char *payload = NULL;
+	unsigned char *payload = NULL;
 	int plen = nfq_get_payload(nfa, &payload);
 	struct nfqnl_msg_packet_hdr *ph = nfq_get_msg_packet_hdr(nfa);
 	TRACE(3, "RECV in queue %d, %s %d, len %d, cnt %u\n",
@@ -274,12 +273,15 @@ packet_cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
 			else
 				pkt->now = tv.tv_sec + (tv.tv_usec > 500000);
 
-			// src address, in network order
+			// IP address, in network order
 			memcpy(&pkt->ipaddr, payload + addr_off, sizeof pkt->ipaddr);
 
 			// id for verdict
 			pkt->id = ntohl(ph->packet_id);
-			TRACE(4, "pkt set: id=%d, src=%s, time=%ld\n",
+
+			// existing mark
+			pkt->old_mark = nfq_get_nfmark(nfa);
+			TRACE(4, "pkt set: cnt=%d, id=%d, IP=%s, time=%ld\n", cnt->cnt,
 				pkt->id, inet_ntoa(*(struct in_addr*)&pkt->ipaddr), pkt->now);
 		}
 	}
@@ -409,7 +411,7 @@ daemon_loop(struct nfq_handle *h, DB *db)
 			continue;
 		}
 
-		TRACE(4, "PACKET RECV size=%d\n", rv);
+		TRACE(4, "\nPACKET RECV size=%d\n", rv);
 		nfq_handle_packet(h, buf, rv); // deliver to queue handlers
 
 		TRACE(4, "%d slot(s) filled...\n", cb_cnt.cnt);
@@ -434,23 +436,24 @@ daemon_loop(struct nfq_handle *h, DB *db)
 						inet_ntoa(*(struct in_addr*)&ip_addr),
 						ip_data.probability);
 				}
-				else if (ip_data.last_block + RECENT_DROP_TIME >= now ||
+				else if ((ip_data.last_block + IPQBDB_RECENT_DROP_TIME >= now &&
+					ip_data.rep_ruling < IPQBDB_REPETITION_MAX) ||
 					ip_data.decay >= IPQBDB_NEVER_DECAY) // static block
 				{
 					verdict = NF_DROP;
 					ip_data.last_block = now;
 					ip_data.block_cnt += 1;
+					ip_data.rep_ruling += 1;
 					need_write = 1;
 					TRACE(2, "BLOCK  %s %s\n",
 						inet_ntoa(*(struct in_addr*)&ip_addr),
-						ip_data.last_block + RECENT_DROP_TIME >= now?
-							"again": "forever");
+						ip_data.decay >= IPQBDB_NEVER_DECAY? "forever": "again");
 				}
-				else // rehabilitate
+				else
 				{
-					// obvious: assert(ip_data.probability > 0);
+					// rehabilitate
 					time_t delta = now - ip_data.last_update;
-					if (UPDATE_TICK_TIME < delta && ip_data.decay > 0.0)
+					if (IPQBDB_UPDATE_TICK_TIME < delta && ip_data.decay > 0.0)
 					{
 						unsigned int const prob =
 							rehabilitated_prob(ip_data.probability,
@@ -463,18 +466,23 @@ daemon_loop(struct nfq_handle *h, DB *db)
 						}
 					}
 
-					if (rand() < ip_data.probability)
+					// rule
+					int const toss = rand();
+					if (toss < ip_data.probability)
 					{
 						verdict = NF_DROP;
 						ip_data.last_block = now;
 						ip_data.block_cnt += 1;
+						ip_data.rep_ruling = 0;
 						need_write = 1;
 					}
+					// else possibly allow rep_ruling to stay above max
 
-					TRACE(2, "%s %s: probability=%.2f%%, delta=%ld, reason=%d\n",
+					TRACE(2, "%s %s: probability=%.2f%%: toss=%.2f, delta=%ld, reason=%d\n",
 						verdict == NF_DROP? "BLOCK ": "ACCEPT",
 						inet_ntoa(*(struct in_addr*)&ip_addr),
 						percent_prob(ip_data.probability),
+						percent_prob(toss),
 						(long)delta, ip_data.reason_id);
 				}
 
@@ -510,10 +518,11 @@ daemon_loop(struct nfq_handle *h, DB *db)
 			}
 			else
 			{
-				rc = nfq_set_verdict_mark(qh, pkt->id, NF_ACCEPT,
+				rc = nfq_set_verdict2(qh, pkt->id, NF_ACCEPT,
 					pkt->qu->mark_value, 0, NULL);
-				TRACE(1, "MARK%2d %s\n",
-					ntohl(pkt->qu->mark_value),
+				TRACE(1, "MARK %#x (was %#x) %s\n",
+					pkt->qu->mark_value,
+					pkt->old_mark,
 					inet_ntoa(*(struct in_addr*)&ip_addr));
 			}
 			if (rc < 0)
