@@ -1,7 +1,7 @@
 /*
 * opendb2.c
 
-Copyright (C) 2008-2013 Alessandro Vesely
+Copyright (C) 2008-2019 Alessandro Vesely
 
 This file is part of Ipqbdb.
 
@@ -25,8 +25,6 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <time.h>
 
-#define __USE_BSD 1
-// for u_int in db.h
 #include <sys/types.h>
 #include <db.h>
 
