@@ -19,7 +19,9 @@ You should have received a copy of the GNU General Public License
 along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 */
+#if ! defined _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 #include <stdio.h>
 #include <syslog.h>
 #include <stdarg.h>
